@@ -22,11 +22,6 @@ extern ClickHouse_Authentication *GloClickHouseAuth;
 #undef max_allowed_packet
 #endif
 
-#if defined(__FreeBSD__) || defined(__APPLE__)
-typedef uint8_t uint8;
-typedef uint8_t uchar;
-#endif
-
 //#define RESULTSET_BUFLEN 16300
 
 #ifndef CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA
@@ -128,7 +123,7 @@ static inline int write_encoded_length_and_string(unsigned char *p, uint64_t val
 	return l+val;
 }
 
-void proxy_compute_sha1_hash_multi(uint8 *digest, const char *buf1, int len1, const char *buf2, int len2) {
+void proxy_compute_sha1_hash_multi(uint8_t *digest, const char *buf1, int len1, const char *buf2, int len2) {
   PROXY_TRACE();
   
   SHA_CTX sha1_context;
@@ -138,7 +133,7 @@ void proxy_compute_sha1_hash_multi(uint8 *digest, const char *buf1, int len1, co
   SHA1_Final(digest, &sha1_context);
 }
 
-void proxy_compute_sha1_hash(uint8 *digest, const char *buf, int len) {
+void proxy_compute_sha1_hash(uint8_t *digest, const char *buf, int len) {
   PROXY_TRACE();
   
   SHA_CTX sha1_context;
@@ -147,13 +142,13 @@ void proxy_compute_sha1_hash(uint8 *digest, const char *buf, int len) {
   SHA1_Final(digest, &sha1_context);
 }
 
-void proxy_compute_two_stage_sha1_hash(const char *password, size_t pass_len, uint8 *hash_stage1, uint8 *hash_stage2) {
+void proxy_compute_two_stage_sha1_hash(const char *password, size_t pass_len, uint8_t *hash_stage1, uint8_t *hash_stage2) {
   proxy_compute_sha1_hash(hash_stage1, password, pass_len);
   proxy_compute_sha1_hash(hash_stage2, (const char *) hash_stage1, SHA_DIGEST_LENGTH);
 }
 
-void proxy_my_crypt(char *to, const uchar *s1, const uchar *s2, uint len) {
-  const uint8 *s1_end= s1 + len;
+void proxy_my_crypt(char *to, const uint8_t *s1, const uint8_t *s2, uint len) {
+  const uint8_t *s1_end= s1 + len;
   while (s1 < s1_end)
     *to++= *s1++ ^ *s2++;
 }
@@ -187,23 +182,23 @@ void unhex_pass(uint8_t *out, const char *in) {
 
 void proxy_scramble(char *to, const char *message, const char *password)
 {
-	uint8 hash_stage1[SHA_DIGEST_LENGTH];
-	uint8 hash_stage2[SHA_DIGEST_LENGTH];
+	uint8_t hash_stage1[SHA_DIGEST_LENGTH];
+	uint8_t hash_stage2[SHA_DIGEST_LENGTH];
 	proxy_compute_two_stage_sha1_hash(password, strlen(password), hash_stage1, hash_stage2);
-	proxy_compute_sha1_hash_multi((uint8 *) to, message, SCRAMBLE_LENGTH, (const char *) hash_stage2, SHA_DIGEST_LENGTH);
-	proxy_my_crypt(to, (const uchar *) to, hash_stage1, SCRAMBLE_LENGTH);
+	proxy_compute_sha1_hash_multi((uint8_t *) to, message, SCRAMBLE_LENGTH, (const char *) hash_stage2, SHA_DIGEST_LENGTH);
+	proxy_my_crypt(to, (const uint8_t *) to, hash_stage1, SCRAMBLE_LENGTH);
 	return;
 }
 
 bool proxy_scramble_sha1(char *pass_reply,  const char *message, const char *sha1_sha1_pass, char *sha1_pass) {
 	bool ret=false;
-	uint8 hash_stage1[SHA_DIGEST_LENGTH];
-	uint8 hash_stage2[SHA_DIGEST_LENGTH];
-	uint8 hash_stage3[SHA_DIGEST_LENGTH];
-	uint8 to[SHA_DIGEST_LENGTH];
+	uint8_t hash_stage1[SHA_DIGEST_LENGTH];
+	uint8_t hash_stage2[SHA_DIGEST_LENGTH];
+	uint8_t hash_stage3[SHA_DIGEST_LENGTH];
+	uint8_t to[SHA_DIGEST_LENGTH];
 	unhex_pass(hash_stage2,sha1_sha1_pass);
-	proxy_compute_sha1_hash_multi((uint8 *) to, message, SCRAMBLE_LENGTH, (const char *) hash_stage2, SHA_DIGEST_LENGTH);
-	proxy_my_crypt((char *)hash_stage1,(const uchar *) pass_reply, to, SCRAMBLE_LENGTH);
+	proxy_compute_sha1_hash_multi((uint8_t *) to, message, SCRAMBLE_LENGTH, (const char *) hash_stage2, SHA_DIGEST_LENGTH);
+	proxy_my_crypt((char *)hash_stage1,(const uint8_t *) pass_reply, to, SCRAMBLE_LENGTH);
 	proxy_compute_sha1_hash(hash_stage3, (const char *) hash_stage1, SHA_DIGEST_LENGTH);
 	if (memcmp(hash_stage2,hash_stage3,SHA_DIGEST_LENGTH)==0) {
 		memcpy(sha1_pass,hash_stage1,SHA_DIGEST_LENGTH);
@@ -1484,6 +1479,54 @@ bool MySQL_Protocol::process_pkt_COM_CHANGE_USER(unsigned char *pkt, unsigned in
 		sha1_pass=NULL;
 	}
 	userinfo->set(NULL,NULL,NULL,NULL); // just to call compute_hash()
+	if (ret) {
+		// we need to process charset if present in CHANGE_USER
+		uint16_t charset=0;
+		int bytes_processed = (db-(char *)pkt);
+		bytes_processed += strlen(db) + 1;
+		int bytes_left = len - bytes_processed;
+		if (bytes_left > 2) {
+			char *p = db;
+			p += strlen(db);
+			p++; // null byte
+			memcpy(&charset, p, sizeof(charset));
+		}
+		// see bug #810
+		if (charset==0) {
+			const MARIADB_CHARSET_INFO *ci = NULL;
+			ci = proxysql_find_charset_name(mysql_thread___default_variables[SQL_CHARACTER_SET]);
+			if (!ci) {
+				proxy_error("Cannot find charset [%s]\n", mysql_thread___default_variables[SQL_CHARACTER_SET]);
+				assert(0);
+			}
+			charset=ci->nr;
+		}
+		// reject connections from unknown charsets
+		const MARIADB_CHARSET_INFO * c = proxysql_find_charset_nr(charset);
+		if (!c) {
+			proxy_error("Client %s:%d is trying to use unknown charset %u. Disconnecting\n", (*myds)->addr.addr, (*myds)->addr.port, charset);
+			proxy_debug(PROXY_DEBUG_MYSQL_AUTH, 5, "Session=%p , DS=%p , user='%s' . Client %s:%d is trying to use unknown charset %u. Disconnecting\n", (*myds), (*myds)->sess, user, (*myds)->addr.addr, (*myds)->addr.port, charset);
+			ret = false;
+			return ret;
+		}
+		assert(sess);
+		assert(sess->client_myds);
+		MySQL_Connection *myconn=sess->client_myds->myconn;
+		assert(myconn);
+
+		myconn->set_charset(charset, CONNECT_START);
+
+		std::stringstream ss;
+		ss << charset;
+
+		/* We are processing handshake from client. Client sends us a character set it will use in communication.
+		 * we store this character set in the client's variables to use later in multiplexing with different backends
+		 */
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_RESULTS, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_CLIENT, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_CHARACTER_SET_CONNECTION, ss.str().c_str());
+		mysql_variables.client_set_value(sess, SQL_COLLATION_CONNECTION, ss.str().c_str());
+	}
 	return ret;
 }
 
@@ -1554,6 +1597,14 @@ bool MySQL_Protocol::process_pkt_handshake_response(unsigned char *pkt, unsigned
 	}
 
 	capabilities     = CPY4(pkt);
+	// see bug #2916. If CLIENT_MULTI_STATEMENTS is set by the client
+	// we enforce setting CLIENT_MULTI_RESULTS, this is the proper and expected
+	// behavior (refer to 'https://dev.mysql.com/doc/c-api/8.0/en/c-api-multiple-queries.html').
+	// Don't enforcing this would cause a mismatch between client and backend
+	// connections flags.
+	if (capabilities & CLIENT_MULTI_STATEMENTS) {
+		capabilities |= CLIENT_MULTI_RESULTS;
+	}
 	(*myds)->myconn->options.client_flag = capabilities;
 	pkt     += sizeof(uint32_t);
 	max_pkt  = CPY4(pkt);
@@ -1910,8 +1961,8 @@ __do_auth:
 						unhex_pass(hash_stage2,sha1_2);
 */
 						proxy_debug(PROXY_DEBUG_MYSQL_AUTH, 5, "Session=%p , DS=%p , username='%s' , session_type=%d\n", (*myds), (*myds)->sess, user, session_type);
-						uint8 hash_stage1[SHA_DIGEST_LENGTH];
-						uint8 hash_stage2[SHA_DIGEST_LENGTH];
+						uint8_t hash_stage1[SHA_DIGEST_LENGTH];
+						uint8_t hash_stage2[SHA_DIGEST_LENGTH];
 						SHA_CTX sha1_context;
 						SHA1_Init(&sha1_context);
 						SHA1_Update(&sha1_context, pass, pass_len);
@@ -2286,7 +2337,6 @@ stmt_execute_metadata_t * MySQL_Protocol::get_binds_from_pkt(void *ptr, unsigned
 }
 
 bool MySQL_Protocol::generate_COM_QUERY_from_COM_FIELD_LIST(PtrSize_t *pkt) {
-	PtrSize_t n_pkt;
 	unsigned int o_pkt_size = pkt->size;
 	char *pkt_ptr = (char *)pkt->ptr;
 
@@ -2312,7 +2362,7 @@ bool MySQL_Protocol::generate_COM_QUERY_from_COM_FIELD_LIST(PtrSize_t *pkt) {
 		(*myds)->com_field_wild=strdup(wild);
 	}
 
-	char *qt = (char *)"SELECT * FROM %s WHERE 1=0";
+	char *qt = (char *)"SELECT * FROM `%s` WHERE 1=0";
 	q = (char *)malloc(strlen(qt)+strlen(tablename));
 	sprintf(q,qt,tablename);
 	l_free(pkt->size, pkt->ptr);
