@@ -294,7 +294,7 @@ void KV_BtreeArray::empty() {
 };
 
 uint64_t Query_Cache::get_data_size_total() {
-	int r=0;
+	uint64_t r=0;
 	int i;
 	for (i=0; i<SHARED_QUERY_CACHE_HASH_TABLES; i++) {
 		r+=KVs[i]->get_data_size();
