@@ -247,6 +247,15 @@ char *mysql_query_digest_and_first_comment(char *s, int _len, char **first_comme
 				flag = 3;
 			}
 
+			else if (*s == '-') {
+				if (prev_char != '-' && i!=(len-1) && ((*(s+1)=='-'))) {
+					flag = 3;
+				}
+				else if (i==0 && ((*(s+1)=='-'))) {
+					flag = 3;
+				}
+			}
+
 			// string - start with '
 			else if(*s == '\'' || *s == '"')
 			{
@@ -307,7 +316,7 @@ char *mysql_query_digest_and_first_comment(char *s, int _len, char **first_comme
 								if (*(s+1) == 'u' || *(s+1) == 'U') {
 									if (*(s+2) == 'l' || *(s+2) == 'L') {
 										if (*(s+3) == 'l' || *(s+3) == 'L') {
-											if (i==len-3) {
+											if (i==len-4) {
 												*p_r++ = '?';
 												*p_r = 0;
 												return r;
