@@ -1,3 +1,13 @@
+/**
+ * @file set_testing-t.cpp
+ * @brief This file tests multiple settings combinations for MySQL variables, and checks that they are
+ *  actually being tracked correctly.
+ * @details The test input is a 'csv' file with name 'set_testing-t.csv'. The file format consists in
+ *  two primary columns which specifies the variables to set (first) and the expected result of setting
+ *  those variables (second), and an optional third column which hold variables that shouldn't be checked
+ *  anymore after the 'SET STATEMENTS' from the same line are executed.
+ */
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,6 +45,7 @@ using nlohmann::json;
 struct TestCase {
 	std::string command;
 	json expected_vars;
+	json reset_vars;
 };
 
 std::vector<TestCase> testCases;
@@ -45,18 +56,27 @@ int readTestCases(const std::string& fileName) {
 	FILE* fp = fopen(fileName.c_str(), "r");
 	if (!fp) return 0;
 
-	char buf[MAX_LINE], col1[MAX_LINE], col2[MAX_LINE];
+	char buf[MAX_LINE], col1[MAX_LINE], col2[MAX_LINE], col3[MAX_LINE] = {0};
 	int n = 0;
 	for(;;) {
 		if (fgets(buf, sizeof(buf), fp) == NULL) break;
-		n = sscanf(buf, " \"%[^\"]\", \"%[^\"]\"", col1, col2);
+		n = sscanf(buf, " \"%[^\"]\", \"%[^\"]\", \"%[^\"]\"", col1, col2, col3);
 		if (n == 0) break;
 
 		char *p = col2;
 		while(*p++) if(*p == '\'') *p = '\"';
 
 		json vars = json::parse(col2);
-		testCases.push_back({col1, vars});
+
+		p = col3;
+		while(col3[0] != 0 && *p++) if(*p == '\'') *p = '\"';
+
+		json reset_vars;
+		if (p != col3) {
+			reset_vars = json::parse(col3);
+		}
+
+		testCases.push_back({col1, vars, reset_vars});
 	}
 
 	fclose(fp);
@@ -159,21 +179,21 @@ void queryVariables(MYSQL *mysql, json& j) {
 	std::stringstream query;
 	if (is_mariadb) {
 		query << "SELECT /* mysql " << mysql << " */ lower(variable_name), variable_value FROM information_schema.session_variables WHERE variable_name IN "
-			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'autocommit', 'sql_auto_is_null', "
+			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'sql_auto_is_null', "
 			" 'sql_safe_updates', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
 			" 'sql_select_limit', 'character_set_results', 'tx_isolation', 'tx_read_only', "
 			" 'sql_auto_is_null', 'collation_connection', 'character_set_connection', 'character_set_client', 'character_set_database', 'group_concat_max_len');";
 	}
 	if (is_cluster) {
 		query << "SELECT /* mysql " << mysql << " */ * FROM performance_schema.session_variables WHERE variable_name IN "
-			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'autocommit', 'sql_auto_is_null', "
+			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'sql_auto_is_null', "
 			" 'sql_safe_updates', 'session_track_gtids', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
 			" 'sql_select_limit', 'character_set_results', 'transaction_isolation', 'transaction_read_only', "
 			" 'sql_auto_is_null', 'collation_connection', 'character_set_connection', 'character_set_client', 'character_set_database', 'wsrep_sync_wait', 'group_concat_max_len');";
 	}
 	if (!is_mariadb && !is_cluster) {
 		query << "SELECT /* mysql " << mysql << " */ * FROM performance_schema.session_variables WHERE variable_name IN "
-			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'autocommit', 'sql_auto_is_null', "
+			" ('hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'sql_auto_is_null', "
 			" 'sql_safe_updates', 'session_track_gtids', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
 			" 'sql_select_limit', 'character_set_results', 'transaction_isolation', 'transaction_read_only', "
 			" 'sql_auto_is_null', 'collation_connection', 'character_set_connection', 'character_set_client', 'character_set_database', 'group_concat_max_len');";
@@ -181,7 +201,7 @@ void queryVariables(MYSQL *mysql, json& j) {
 	//fprintf(stderr, "TRACE : QUERY 3 : variables %s\n", query.str().c_str());
 	if (mysql_query(mysql, query.str().c_str())) {
 		if (silent==0) {
-			fprintf(stderr,"%s\n", mysql_error(mysql));
+			fprintf(stderr,"ERROR while running -- \"%s\" :  (%d) %s\n", query.str().c_str(), mysql_errno(mysql), mysql_error(mysql));
 		}
 	} else {
 		MYSQL_RES *result = mysql_store_result(mysql);
@@ -198,7 +218,7 @@ void queryInternalStatus(MYSQL *mysql, json& j) {
 	//fprintf(stderr, "TRACE : QUERY 4 : variables %s\n", query);
 	if (mysql_query(mysql, query)) {
 		if (silent==0) {
-			fprintf(stderr,"%s\n", mysql_error(mysql));
+			fprintf(stderr,"ERROR while running -- \"%s\" :  (%d) %s\n", query, mysql_errno(mysql), mysql_error(mysql));
 		}
 	} else {
 		MYSQL_RES *result = mysql_store_result(mysql);
@@ -224,7 +244,7 @@ void queryInternalStatus(MYSQL *mysql, json& j) {
 				j["conn"]["sql_log_bin"] = "OFF";
 			}
 
-			// autocommit {true|false}
+			// sql_auto_is_null {true|false}
 			if (!el.value()["sql_auto_is_null"].dump().compare("ON") ||
 					!el.value()["sql_auto_is_null"].dump().compare("1") ||
 					!el.value()["sql_auto_is_null"].dump().compare("on") ||
@@ -240,6 +260,8 @@ void queryInternalStatus(MYSQL *mysql, json& j) {
 				j["conn"]["sql_auto_is_null"] = "OFF";
 			}
 
+			// completely remove autocommit test
+/*
 			// autocommit {true|false}
 			if (!el.value()["autocommit"].dump().compare("ON") ||
 					!el.value()["autocommit"].dump().compare("1") ||
@@ -255,7 +277,7 @@ void queryInternalStatus(MYSQL *mysql, json& j) {
 				el.value().erase("autocommit");
 				j["conn"]["autocommit"] = "OFF";
 			}
-
+*/
 			// sql_safe_updates
 			if (!el.value()["sql_safe_updates"].dump().compare("\"ON\"") ||
 					!el.value()["sql_safe_updates"].dump().compare("\"1\"") ||
@@ -361,6 +383,37 @@ void queryInternalStatus(MYSQL *mysql, json& j) {
 	}
 }
 
+std::vector<std::string> forgotten_vars {};
+
+/**
+ * @brief Checks that after setting 'session_track_gtids', the new set value follows ProxySQL rules
+ * for this particular variable. This is:
+ * - backend connections are by default set to `mysql-default_session_track_gtids`.
+ * - if `mysql-default_session_track_gtids=OFF` (the default), `session_track_gtids` is not changed on backend.
+ * - if the client asks for `session_track_gtids=OFF`, proxysql ignores it (it just acknowledge it).
+ * - if the client asks for `session_track_gtids=OWN_GTID`, proxysql will apply it.
+ * - if the client asks for `session_track_gtids=ALL_GTIDS`, proxysql will switch to OWN_GTID and generate a warning.
+ * - if the backend doesn't support `session_track_gtids` (for example in MySQL 5.5 and MySQL 5.6), proxysql won't apply it. It knows checking server capabilities
+ *
+ * @param expVal The value to which 'session_track_gtids' have been set.
+ * @param sVal The ProxySQL session value for 'session_track_gtids'.
+ * @param mVal The MySQL session value for 'session_track_gtids'.
+ * @return True in case the check succeed, false otherwise.
+ */
+bool check_session_track_gtids(const std::string& expVal, const std::string& sVal, const std::string& mVal) {
+	bool res = false;
+
+	if (expVal == "OFF") {
+		res = expVal == sVal;
+	} else if (expVal == "OWN_GTID" && (sVal == mVal && sVal == "OWN_GTID")) {
+		res = true;
+	} else if (expVal == "ALL_GTIDS" && (sVal == mVal && sVal == "OWN_GTID")) {
+		res = true;
+	}
+
+	return res;
+}
+
 void * my_conn_thread(void *arg) {
 	g_seed = time(NULL) ^ getpid() ^ pthread_self();
 	unsigned int select_OK=0;
@@ -408,12 +461,23 @@ void * my_conn_thread(void *arg) {
 			mysql=mysqlconns[r1];
 			vars = varsperconn[r1];
 		}
-
+		if (strcmp(username,(char *)"root")) {
+			if (strstr(testCases[r2].command.c_str(),"database")) {
+				std::lock_guard<std::mutex> lock(mtx_);
+				skip(1, "mysql connection [%p], command [%s]", mysql, testCases[r2].command.c_str());
+				continue;
+			}
+			if (strstr(testCases[r2].command.c_str(),"sql_log_bin")) {
+				std::lock_guard<std::mutex> lock(mtx_);
+				skip(1, "mysql connection [%p], command [%s]", mysql, testCases[r2].command.c_str());
+				continue;
+			}
+		}
 		std::vector<std::string> commands = split(testCases[r2].command.c_str(), ';');
 		for (auto c : commands) {
 			if (mysql_query(mysql, c.c_str())) {
 				if (silent==0) {
-					fprintf(stderr,"%s\n", mysql_error(mysql));
+					fprintf(stderr,"ERROR while running -- \"%s\" :  (%d) %s\n", c.c_str(), mysql_errno(mysql), mysql_error(mysql));
 				}
 			} else {
 				MYSQL_RES *result = mysql_store_result(mysql);
@@ -475,11 +539,24 @@ void * my_conn_thread(void *arg) {
 		json proxysql_vars;
 		queryInternalStatus(mysql, proxysql_vars);
 
+		if (!testCases[r2].reset_vars.empty()) {
+			for (const auto& var : testCases[r2].reset_vars) {
+				if (std::find(forgotten_vars.begin(), forgotten_vars.end(), var) == forgotten_vars.end()) {
+					forgotten_vars.push_back(var);
+				}
+			}
+		}
+
 		bool testPassed = true;
+		int variables_tested = 0;
 		for (auto& el : vars.items()) {
 			auto k = mysql_vars.find(el.key());
 			auto s = proxysql_vars["conn"].find(el.key());
 
+			if (std::find(forgotten_vars.begin(), forgotten_vars.end(), el.key()) != forgotten_vars.end()) {
+				continue;
+			}
+
 			if (k == mysql_vars.end())
 				fprintf(stderr, "Variable %s->%s in mysql resultset was not found.\nmysql data : %s\nproxysql data: %s\ncsv data %s\n",
 						el.value().dump().c_str(), el.key().c_str(), mysql_vars.dump().c_str(), proxysql_vars.dump().c_str(), vars.dump().c_str());
@@ -488,18 +565,24 @@ void * my_conn_thread(void *arg) {
 				fprintf(stderr, "Variable %s->%s in proxysql resultset was not found.\nmysql data : %s\nproxysql data: %s\ncsv data %s\n",
 						el.value().dump().c_str(), el.key().c_str(), mysql_vars.dump().c_str(), proxysql_vars.dump().c_str(), vars.dump().c_str());
 
-			if (k.value() != el.value() || s.value() != el.value()) {
+			if (
+				(el.key() != "session_track_gtids" && (k.value() != el.value() || s.value() != el.value())) ||
+				(el.key() == "session_track_gtids" && !check_session_track_gtids(el.value(), s.value(), k.value()))
+			) {
 				__sync_fetch_and_add(&g_failed, 1);
 				testPassed = false;
 				fprintf(stderr, "Test failed for this case %s->%s.\n\nmysql data %s\n\n proxysql data %s\n\n csv data %s\n\n\n",
 						el.value().dump().c_str(), el.key().c_str(), mysql_vars.dump().c_str(), proxysql_vars.dump().c_str(), vars.dump().c_str());
 				ok(testPassed, "mysql connection [%p], thread_id [%lu], command [%s]", mysql, mysql->thread_id, testCases[r2].command.c_str());
-				exit(0);
+				// In case of failing test, exit completely.
+				exit(EXIT_FAILURE);
+			} else {
+				variables_tested++;
 			}
 		}
 		{
 			std::lock_guard<std::mutex> lock(mtx_);
-			ok(testPassed, "mysql connection [%p], thread_id [%lu], command [%s]", mysql, mysql->thread_id, testCases[r2].command.c_str());
+			ok(testPassed, "mysql connection [%p], thread_id [%lu], variables_tested [%d], command [%s]", mysql, mysql->thread_id, variables_tested, testCases[r2].command.c_str());
 		}
 	}
 	__sync_fetch_and_add(&query_phase_completed,1);
@@ -515,6 +598,8 @@ int main(int argc, char *argv[]) {
 		return exit_status();
 
 	std::string fileName(std::string(cl.workdir) + "/set_testing-t.csv");
+/*
+	// do not connect to admin at all
 	MYSQL* mysqladmin = mysql_init(NULL);
 	if (!mysqladmin)
 		return exit_status();
@@ -534,8 +619,7 @@ int main(int argc, char *argv[]) {
 	MYSQL_QUERY(mysqladmin, "update global_variables set variable_value='true' where variable_name='mysql-enforce_autocommit_on_reads'");
 	MYSQL_QUERY(mysqladmin, "load mysql variables to runtime");
 
-	mysql_close(mysqladmin);
-
+*/
 	MYSQL* mysql = mysql_init(NULL);
 	if (!mysql)
 		return exit_status();
@@ -604,4 +688,4 @@ int main(int argc, char *argv[]) {
 		pthread_join(thi[i], NULL);
 	}
 	return exit_status();
-}
+}
\ No newline at end of file
